#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time
import logging

_logger = logging.getLogger(__name__)
ENABLE_DEBUG_LOG = False

ARGON_STARTUP_DELAY = 5
ARGON_SHUTDOWN_DELAY = 45       # Anforderung Hr. Apfel vom 07.05.2019


class ArgonControl(object):
    """!
    Unterdruckerzeugung und Unterdruckregelung
    """

    def __init__(self, daio, analog_input, config_manager, error_handler):
        """!
        Initialisiert die Steuerung der Argonflutung .
        @param daio: Instanz der DIO
        @param analog_input: Instanz der AIO
        @param config_manager: Instanz des Config Managers
        @param error_handler: Instanz des Fehlermanagers
        """
        _logger.debug("ArgonControl init")
        self._name = "ArgonControl"

        self._daio = daio
        self._analog_input = analog_input
        self._error_handler = error_handler
        self._argon_valve_open = False
        self._argon_startup_timer = time.time() + ARGON_STARTUP_DELAY
        self._argon_shutdown_timer = time.time() + ARGON_SHUTDOWN_DELAY

        # Argon verfügbar = 1, Argon nicht verfügbar = 0
        self._argon_enabled = True if int(config_manager.get_config_file_value('argon_control', 'argon_enabled')) == 1 else False
        # Argon: minimal benötigter Druck nach Druckminderer [mbar]
        self._argon_min_pressure = int(config_manager.get_config_file_value('argon_control', 'argon_min_pressure'))
        # Argon: maximal erlaubter Druck nach Druckminderer [mbar]
        self._argon_max_pressure = int(config_manager.get_config_file_value('argon_control', 'argon_max_pressure'))
        # Argon Ventil offen: minimaler Druck nach Argon-Düse zur Messung, ob Durchfluss i.O. [mbar]
        self._argon_min_flow_pressure = int(config_manager.get_config_file_value('argon_control', 'argon_min_flow_pressure'))
        # Argon Ventil geschlossen: maximal erlaubter Druck nach Argon-Düse zur Messung, ob Ventil dicht ist [mbar]
        self._argon_leakage_pressure = int(config_manager.get_config_file_value('argon_control', 'argon_leakage_pressure'))

        self._argon_monitor_state = "IDLE"

        self._argon_input_pressure_low = False
        self.disableArgon()

    def enableArgon(self):
        """!
        Argonflutung aktivieren
        """
        self._daio.valve_argon(True)        # 24V Argon Ventil auf
        self._daio.rel_res(True)            # Auf Wunsch von Herrn Apfel auch 230V Ventil möglich
        self._argon_startup_timer = time.time() + ARGON_STARTUP_DELAY
        self._argon_valve_open = True

    def disableArgon(self):
        """!
        Argonflutung deaktivieren
        """
        self._daio.valve_argon(False)       # 24V Argon Ventil zu
        self._daio.rel_res(False)           # Auf Wunsch von Herrn Apfel auch 230V Ventil möglich
        self._argon_shutdown_timer = time.time() + ARGON_SHUTDOWN_DELAY
        self._argon_valve_open = False

    def getArgonAvailable(self):
        """!
        Rückmeldung, ob Argonflutung verfügbar oder nicht
        @return True - Argonflutung verfügbar, False - Argonflutung nicht verfügbar
        """
        return self._argon_enabled

    def getArgonPressureLow(self):
        """!
        Zustand des Flaschendrucks
        @return True - Flaschendruck niedrig, False - Flaschendruck ausreichend
        """
        return self._argon_input_pressure_low

    def getArgonError(self):
        """!
        Zustand des Argonsystems
        @return True - Fehler im Argonsystem liegt vor, False - Argonsystem ist fehlerfrei
        """
        if (self._error_handler.is_error_active("F8000") or
                self._error_handler.is_error_active("F80") or
                self._error_handler.is_error_active("F81") or
                self._error_handler.is_error_active("F82")):
            return True
        else:
            return False

    def resetArgonError(self):
        """!
        Setzt die Fehler des Argonsystems zurück.
        """
        self._error_handler.reset_error("F8000")
        self._error_handler.reset_error("F80")
        self._error_handler.reset_error("F81")
        self._error_handler.reset_error("F82")

    def __call__(self):
        """!
        Funktion Überwacht das Argonsystem.
        Sie erzeugt Fehlermeldungen, wenn der Eingangsdruck zu niedrig oder zu hoch ist.
        Sie erzeugt eine Fehlermeldung, wenn der Durchfluss zu niedrig ist.
        Muss zyklisch aufgerufen werden.
        """
        if self._argon_enabled:         # nur Überwachen, wenn Argon im Configfile aktiv geschaltet wurde
            argon_pressure = self._analog_input.pressure_sensor_1   # Drucksensor 1 lesen
            argon_flow_pressure = self._analog_input.pressure_sensor_2   # Drucksensor 2 lesen

            self._argon_input_pressure_low = False

            if self._argon_monitor_state == "IDLE":
                # Wenn das Ventil geöffnet wurde, dann warten bis das System geflutet ist.
                if self._argon_valve_open:
                    self._error_handler.reset_error("F8000")
                    self._error_handler.reset_error("F80")
                    if self._argon_startup_timer < time.time():
                        self._argon_monitor_state = "STARTUP"

            elif self._argon_monitor_state == "STARTUP":
                # Wenn beim Start die Flasche fehlerhaft ist, kann sofort abgebrochen werden mit F80
                if argon_pressure < self._argon_min_pressure:
                    # Flaschendruck zu niedrig, Fehlermeldung
                    if not self._error_handler.is_error_active("F80"):
                        _logger.debug("argon pressure too low: {:5.0f} mbar".format(argon_pressure))
                        self._error_handler.set_error("F80")  # Argon Eingangsdruck falsch
                    self._argon_input_pressure_low = True
                elif argon_pressure > self._argon_max_pressure:
                    # Flaschendruck zu hoch, Fehlermeldung
                    if not self._error_handler.is_error_active("F80"):
                        _logger.debug("argon pressure too high: {:5.0f} mbar".format(argon_pressure))
                        self._error_handler.set_error("F80")  # Argon Eingangsdruck falsch

                self._argon_monitor_state = "ACTIVE"

            elif self._argon_monitor_state == "ACTIVE":
                if argon_pressure < self._argon_min_pressure:
                    # Flaschendruck zu niedrig, Fehlermeldung
                    if not (self._error_handler.is_error_active("F8000") or self._error_handler.is_error_active("F80")):
                        _logger.debug("argon pressure too low: {:5.0f} mbar".format(argon_pressure))
                        self._error_handler.set_error("F8000")  # Argon Eingangsdruck falsch
                    self._argon_input_pressure_low = True
                elif argon_pressure > self._argon_max_pressure:
                    # Flaschendruck zu hoch, Fehlermeldung
                    if not (self._error_handler.is_error_active("F8000") or self._error_handler.is_error_active("F80")):
                        _logger.debug("argon pressure too high: {:5.0f} mbar".format(argon_pressure))
                        self._error_handler.set_error("F8000")  # Argon Eingangsdruck falsch

                if argon_flow_pressure < self._argon_min_flow_pressure and self._argon_valve_open:
                    # Kein Durchfluss, Fehlermeldung
                    if not (self._error_handler.is_error_active("F8000") or
                            self._error_handler.is_error_active("F80") or
                            self._error_handler.is_error_active("F81")):
                        _logger.debug("argon flow pressure too low: {:5.0f} mbar".format(argon_flow_pressure))
                        self._error_handler.set_error("F81")  # Argondurchfluss fehlerhaft

                if not self._argon_valve_open and self._argon_shutdown_timer < time.time():
                    self._argon_monitor_state = "SHUTDOWN"

            elif self._argon_monitor_state == "SHUTDOWN":
                if argon_flow_pressure > self._argon_leakage_pressure:
                    # Kein Durchfluss, Fehlermeldung
                    if not (self._error_handler.is_error_active("F8000") or
                            self._error_handler.is_error_active("F80") or
                            self._error_handler.is_error_active("F82")):
                        _logger.debug("argon valve leakage detected: {:5.0f} mbar".format(argon_flow_pressure))
                        self._error_handler.set_error("F82")  # Argon-Ventil defekt (Leckage)

                self._argon_monitor_state = "IDLE"
